@@ -1637,56 +1637,6 @@ static void test_append_locked(abts_case *tc, void *data)
     apr_file_remove(fname, p);
 }
 
-static void test_append_read(abts_case *tc, void *data)
-{
-    apr_status_t rv;
-    apr_file_t *f;
-    const char *fname = "data/testappend_read.dat";
-    apr_off_t offset;
-    char buf[64];
-
-    apr_file_remove(fname, p);
-
-    /* Create file with contents. */
-    rv = apr_file_open(&f, fname, APR_FOPEN_WRITE | APR_FOPEN_CREATE,
-                       APR_FPROT_OS_DEFAULT, p);
-    APR_ASSERT_SUCCESS(tc, "create file", rv);
-
-    rv = apr_file_puts("abc", f);
-    APR_ASSERT_SUCCESS(tc, "write to file", rv);
-    apr_file_close(f);
-
-    /* Re-open it with APR_FOPEN_APPEND. */
-    rv = apr_file_open(&f, fname,
-                       APR_FOPEN_READ | APR_FOPEN_WRITE | APR_FOPEN_APPEND,
-                       APR_FPROT_OS_DEFAULT, p);
-    APR_ASSERT_SUCCESS(tc, "open file", rv);
-
-    /* Test the initial file offset.  Even though we used APR_FOPEN_APPEND,
-     * the offset should be kept in the beginning of the file until the
-     * first append.  (Previously, the Windows implementation performed
-     * an erroneous seek to the file's end right after opening it.)
-     */
-    offset = 0;
-    rv = apr_file_seek(f, APR_CUR, &offset);
-    APR_ASSERT_SUCCESS(tc, "get file offset", rv);
-    ABTS_INT_EQUAL(tc, 0, (int)offset);
-
-    /* Test reading from the file. */
-    rv = apr_file_gets(buf, sizeof(buf), f);
-    APR_ASSERT_SUCCESS(tc, "read from file", rv);
-    ABTS_STR_EQUAL(tc, "abc", buf);
-
-    /* Test the file offset after reading. */
-    offset = 0;
-    rv = apr_file_seek(f, APR_CUR, &offset);
-    APR_ASSERT_SUCCESS(tc, "get file offset", rv);
-    ABTS_INT_EQUAL(tc, 3, (int)offset);
-
-    apr_file_close(f);
-    apr_file_remove(fname, p);
-}
-
 static void test_large_write_buffered(abts_case *tc, void *data)
 {
     apr_status_t rv;
@@ -2227,6 +2177,50 @@ static void test_read_buffered_seek(abts_case *tc, void *data)
     apr_file_remove(fname, p);
 }
 
+static void test_append_buffered(abts_case *tc, void *data)
+{
+    apr_status_t rv;
+    apr_file_t *f;
+    const char *fname = "data/testappend_buffered.dat";
+    apr_size_t bytes_written;
+    char buf[64];
+
+    apr_file_remove(fname, p);
+
+    /* Create file with contents. */
+    rv = apr_file_open(&f, fname, APR_FOPEN_WRITE | APR_FOPEN_CREATE,
+                       APR_FPROT_OS_DEFAULT, p);
+    APR_ASSERT_SUCCESS(tc, "create file", rv);
+
+    rv = apr_file_write_full(f, "abc", 3, &bytes_written);
+    APR_ASSERT_SUCCESS(tc, "write to file", rv);
+    apr_file_close(f);
+
+    /* Re-open it with APR_FOPEN_APPEND and APR_FOPEN_BUFFERED. */
+    rv = apr_file_open(&f, fname,
+                       APR_FOPEN_READ | APR_FOPEN_WRITE |
+                       APR_FOPEN_APPEND | APR_FOPEN_BUFFERED,
+                       APR_FPROT_OS_DEFAULT, p);
+    APR_ASSERT_SUCCESS(tc, "open file", rv);
+
+    /* Append to the file. */
+    rv = apr_file_write_full(f, "def", 3, &bytes_written);
+    APR_ASSERT_SUCCESS(tc, "write to file", rv);
+    apr_file_close(f);
+
+    rv = apr_file_open(&f, fname, APR_FOPEN_READ,
+                       APR_FPROT_OS_DEFAULT, p);
+    APR_ASSERT_SUCCESS(tc, "open file", rv);
+
+    /* Test the file contents. */
+    rv = apr_file_gets(buf, sizeof(buf), f);
+    APR_ASSERT_SUCCESS(tc, "read from file", rv);
+    ABTS_STR_EQUAL(tc, "abcdef", buf);
+
+    apr_file_close(f);
+    apr_file_remove(fname, p);
+}
+
 abts_suite *testfile(abts_suite *suite)
 {
     suite = ADD_SUITE(suite)
@@ -2280,7 +2274,6 @@ abts_suite *testfile(abts_suite *suite)
     abts_run_test(suite, test_datasync_on_stream, NULL);
     abts_run_test(suite, test_atomic_append, NULL);
     abts_run_test(suite, test_append_locked, NULL);
-    abts_run_test(suite, test_append_read, NULL);
     abts_run_test(suite, test_large_write_buffered, NULL);
     abts_run_test(suite, test_two_large_writes_buffered, NULL);
     abts_run_test(suite, test_small_and_large_writes_buffered, NULL);
@@ -2292,6 +2285,7 @@ abts_suite *testfile(abts_suite *suite)
     abts_run_test(suite, test_read_buffered_spanning_over_bufsize, NULL);
     abts_run_test(suite, test_single_byte_reads_buffered, NULL);
     abts_run_test(suite, test_read_buffered_seek, NULL);
+    abts_run_test(suite, test_append_buffered, NULL);
 
     return suite;
 }
